# -*- encoding: utf-8 -*-
import os
import sys
import time

import boto3
from halo import Halo

from suite_py.lib import logger


class Aws:
    def __init__(self, config):
        self._profile = ""
        if "profile" in config.aws:
            self._profile = config.aws["profile"]
        if os.getenv("AWS_PROFILE", None):
            self._profile = os.getenv("AWS_PROFILE", None)

        if not self._profile:
            logger.warning(
                "Please update your config file:\n'aws:\n    profile: <aws profile name>' "
            )
            logger.warning("Or set 'AWS_PROFILE=<aws profile name>'")
            sys.exit(-1)

        self._session = boto3.session.Session(profile_name=self._profile)
        self._cloudformation = self._session.client(
            "cloudformation", region_name="eu-west-1"
        )
        self._cloudfront = self._session.client("cloudfront", region_name="eu-west-1")
        self._ecs = self._session.client("ecs", region_name="eu-west-1")
        self._ec2 = self._session.client("ec2", region_name="eu-west-1")

    def update_stack(self, stack_name, version):
        params = self.update_template_params(stack_name, version)
        if params:
            logger.info(f"Updating cloudformation template {stack_name}")
            self._cloudformation.update_stack(
                StackName=stack_name,
                TemplateBody=self.get_template_from_stack_name(stack_name),
                Parameters=params,
                RoleARN="arn:aws:iam::001575623345:role/rollback-service-role-clo-CloudformationRollbackRo-MTC7E4N9PXOK",
            )
        else:
            logger.error(f"No parameter found on the stack {stack_name}")
            sys.exit(-1)

    def update_template_params(self, stack_name, version):
        new_parameter = []
        ReleaseVersionFound = False
        current_parameters = self.get_parameters_from_stack_name(stack_name)

        for param in current_parameters:
            if param["ParameterKey"] == "ReleaseVersion":
                new_parameter.append(
                    {"ParameterKey": param["ParameterKey"], "ParameterValue": version}
                )
                ReleaseVersionFound = True
            else:
                new_parameter.append(
                    {"ParameterKey": param["ParameterKey"], "UsePreviousValue": True}
                )

        if ReleaseVersionFound:
            return new_parameter
        return None

    def get_parameters_from_stack_name(self, stack_name):
        return self._cloudformation.describe_stacks(StackName=stack_name)["Stacks"][0][
            "Parameters"
        ]

    def get_template_from_stack_name(self, stack_name):
        return self._cloudformation.get_template(StackName=stack_name)["TemplateBody"]

    def get_stacks_name(self, repo):
        stacks_name = []
        base_stacks_name = [
            f"ecs-task-{repo}-production",
            f"ecs-task-{repo}-vpc-production",
            f"ecs-job-{repo}-production",
            f"batch-job-{repo}-production",
        ]

        for stack_name in base_stacks_name:
            if self.stack_exists(stack_name):
                stacks_name.append(stack_name)

        return stacks_name

    def stack_exists(self, stack_name):
        try:
            self._cloudformation.describe_stacks(StackName=stack_name)
            return True
        except Exception:
            return False

    def get_stack_status(self, stack_name):
        return self._cloudformation.describe_stacks(StackName=stack_name)["Stacks"][0][
            "StackStatus"
        ]

    def get_artifacts_from_s3(self, bucket_name, prefix):
        # pylint has issues with boto3...
        # pylint: disable=no-member
        with Halo(text="Loading releases...", spinner="dots", color="magenta"):
            s3 = boto3.resource("s3")
            bucket_artifacts = s3.Bucket(bucket_name)
            artifacts = []

            for bucket_object in bucket_artifacts.objects.filter(Prefix=prefix):
                artifacts.append(bucket_object.key.replace(prefix, ""))

        return artifacts

    def is_cloudfront_distribution(self, repo):
        with Halo(text="Loading releases...", spinner="dots", color="magenta"):
            distributions = self._cloudfront.list_distributions()["DistributionList"][
                "Items"
            ]
            for distribution in distributions:
                if (
                    f"prima-prod-{repo}.s3.amazonaws.com"
                    == distribution["Origins"]["Items"][0]["DomainName"]
                ):
                    return True
        return False

    def get_tag_from_s3_artifact(self, bucket_name, prefix, artifact):
        s3 = boto3.client("s3")

        return s3.get_object_tagging(Bucket=bucket_name, Key=f"{prefix}{artifact}")[
            "TagSet"
        ]

    def stack_update_completed(self, stack_name):
        with Halo(text="Rollback in progress...", spinner="dots", color="magenta"):
            for _ in range(0, 60):
                stack_status = self.get_stack_status(stack_name)
                if stack_status == "UPDATE_COMPLETE":
                    return True
                if "FAILED" in stack_status or "ROLLBACK" in stack_status:
                    break
                time.sleep(10)
        return False

    def update_stacks(self, stacks_name, version):
        for stack_name in stacks_name:
            self.update_stack(stack_name, version)

        for stack_name in stacks_name:
            if not self.stack_update_completed(stack_name):
                logger.error(
                    f"Error updating CloudFormation stack {stack_name}. Contact the DevOps team."
                )
                sys.exit(-1)

    def get_ecs_services(self, cluster_name):
        with Halo(text="Loading services...", spinner="dots", color="magenta"):
            done = False
            services = []
            options = {"cluster": cluster_name, "maxResults": 10}
            nextToken = None

            while not done:
                if nextToken:
                    options["nextToken"] = nextToken
                list_services = self._ecs.list_services(**options)

                if (
                    "nextToken" in list_services.keys()
                    and list_services["nextToken"] != "null"
                ):
                    nextToken = list_services["nextToken"]
                else:
                    done = True

                if not list_services["serviceArns"]:
                    return []

                list_services_description = self._ecs.describe_services(
                    cluster=cluster_name, services=list_services["serviceArns"]
                )

                services = services + list_services_description["services"]

            return services

    def get_ids_from_container_instances(self, cluster_name, container_instances):
        ids = []

        container_instances_description = self._ecs.describe_container_instances(
            cluster=cluster_name, containerInstances=container_instances
        )

        for c in container_instances_description["containerInstances"]:
            ids.append(c["ec2InstanceId"])

        return ids

    def get_ips_from_container_instances(self, cluster_name, container_instances):
        instances_id = []
        ips = []

        container_instances_description = self._ecs.describe_container_instances(
            cluster=cluster_name, containerInstances=container_instances
        )

        for c in container_instances_description["containerInstances"]:
            instances_id.append(c["ec2InstanceId"])

        instances_description = self._ec2.describe_instances(InstanceIds=instances_id)

        for reservation in instances_description["Reservations"]:
            for instance in reservation["Instances"]:
                ips.append(instance["PrivateIpAddress"])

        return ips

    def get_container_instances_arn_from_service(self, cluster_name, service_name):
        container_instances_arn = []
        tasks = self._ecs.list_tasks(
            cluster=cluster_name,
            desiredStatus="RUNNING",
            serviceName=service_name,
        )["taskArns"]

        if tasks:
            tasks_descriptions = self._ecs.describe_tasks(
                cluster=cluster_name, tasks=tasks
            )

            for task in tasks_descriptions["tasks"]:
                container_instances_arn.append(task["containerInstanceArn"])

        return container_instances_arn

    def get_ecs_clusters(self, env):
        clusters = []
        list_clusters = self._ecs.list_clusters()
        for c in list_clusters["clusterArns"]:
            if f"-{env}-" in c and "-ci-" not in c:
                clusters.append(c)
        return clusters



from __future__ import annotations

import re
import sys
from dataclasses import dataclass
from pathlib import Path
from typing import Any, Dict, List, Optional, cast

from yaml import safe_load

from suite_py.lib import logger
from suite_py.lib.handler.changelog_handler import ChangelogHandler
from suite_py.lib.handler.git_handler import GitHandler
from suite_py.lib.handler.github_handler import GithubHandler
from suite_py.lib.handler.version_handler import VersionHandler


class Bump:
    """
    Bumps the local project version.
    """

    # pylint: disable=too-many-instance-attributes
    def __init__(self, project, config, tokens):
        self._changelog_handler = ChangelogHandler()
        _github = GithubHandler(tokens)
        self._repo = _github.get_repo(project)
        _git = GitHandler(project, config)
        self._version = VersionHandler(_repo=self._repo, _git=_git, _github=_github)

    def run(self, project: Optional[str] = None, version: Optional[str] = None) -> None:
        configurations = read_bump_configs_from_yaml(Path(".versions.yml"))
        version_config = (
            configurations[project] if project else configurations["base_project"]
        )

        current_version = self._find_current_version(version_config)

        logger.info(  # pylint: disable=logging-fstring-interpolation
            f"The current version is {current_version}"
        )
        selected_version = version or self._version.select_new_version(
            current_version, allow_prerelease=True, allow_custom_version=True
        )
        self._replace_version_in_project_files(
            current_version, cast(str, selected_version), version_config
        )

        if self._changelog_handler.changelog_exists():
            self._changelog_handler.bump_changelog(
                self._repo.html_url, current_version, selected_version
            )

        logger.info(f"Successfully bumped to version {selected_version}")

    @staticmethod
    def _find_current_version(config: BumpVersionConfig) -> str:
        versions = []
        for file_config in config.files:
            pattern = re.compile(
                file_config.pattern.format(
                    version=r"([0-9]+\.[0-9]+\.[0-9]+(?:-\w+\.[0-9]+)?)"
                ),
                flags=re.MULTILINE,
            )
            matches = pattern.findall(file_config.path.read_text(encoding="utf-8"))
            if len(matches) == 0:
                logger.error(
                    f'Cannot find "{file_config.pattern}" in file {str(file_config.path.absolute())}'
                )
                sys.exit(-1)
            versions += matches

        if len(set(versions)) != 1:
            logger.error(
                "Some of the versions inside the configured project files do not match."
            )
            sys.exit(-1)

        return versions[0]

    def _replace_version_in_project_files(
        self, current_version: str, target_version: str, config: BumpVersionConfig
    ) -> None:
        """
        Replaces current_version with target_version everywhere
        """
        for file_config in config.files:
            search = file_config.pattern.format(
                version=rf"(?P<version>{current_version})"
            )
            self._replace_in_file(file_config.path, search, target_version)

    @staticmethod
    def _replace_in_file(filepath: Path, search: str, target_version: str) -> None:
        """
        Replaces the named group `version` of `search` regex with `target_version` inside `filepath`.
        """
        text = filepath.read_text(encoding="utf-8")

        matched = re.search(search, text, re.MULTILINE)
        if matched is None:
            logger.error(f"No version matched in the provided file: {filepath}")
            sys.exit(-1)

        matched_from, matched_to = matched.span("version")
        final_text = text[:matched_from] + target_version + text[matched_to:]

        filepath.write_text(final_text, encoding="utf-8")


def read_bump_configs_from_yaml(filepath: Path) -> Dict[str, BumpVersionConfig]:
    with open(filepath, "r", encoding="utf-8") as yaml_file:
        content = safe_load(yaml_file)
    configs = {}
    if "files" in content:
        configs["base_project"] = BumpVersionConfig.from_dict(content)
    if "projects" in content:
        for name, subconfig in content["projects"].items():
            configs[name] = BumpVersionConfig.from_dict(subconfig)
    return configs


@dataclass
class BumpVersionConfig:
    """
    Represents the YAML configuration file for the Bump command.
    """

    files: List[VersionFileConfig]
    """
    The list of files in which the version should be bumped.
    """

    @classmethod
    def from_dict(cls, values: Dict[str, Any]) -> BumpVersionConfig:
        return cls(
            files=[
                VersionFileConfig(
                    path=Path(file_config["path"]), pattern=file_config["pattern"]
                )
                for file_config in values["files"]
            ]
        )


@dataclass
class VersionFileConfig:
    """
    Configuration for a single file in which the version should be bumped.
    """

    path: Path
    """
    Path to the file, relative from the project root.
    """
    pattern: str
    """
    The pattern used to lookup and replace the version.
    """



# -*- encoding: utf-8 -*-
import logging

import logzero
from logzero import logger as _logger

DEFAULT_FORMAT = "%(color)s[%(levelname)1.4s]%(end_color)s %(message)s"
# Set a custom formatter
formatter = logzero.LogFormatter(fmt=DEFAULT_FORMAT)
logzero.setup_default_logger(formatter=formatter)
_logger.setLevel(logging.INFO)


def setLevel(level):
    _logger.setLevel(level)


def debug(message):
    _logger.debug(message)


def info(message):
    _logger.info(message)


def warning(message):
    _logger.warning(message)


def error(message):
    _logger.error(message)
